@@ -15,7 +15,7 @@ use VM::HetznerCloud::Schema;
 
 use Mojo::Base -strict, -signatures;
 
-our $VERSION = '0.0.2'; # VERSION
+our $VERSION = '0.0.3'; # VERSION
 
 has token    => ( is => 'ro', isa => Str, required => 1 );
 has host     => ( is => 'ro', isa => MojoURL["https?"], default => sub { 'https://api.hetzner.cloud' }, coerce => 1 );
@@ -31,8 +31,16 @@ has client   => (
 );
 
 sub _request ( $self, $partial_uri, $params = {}, $param_def = {}, $opts = {} ) {
-    if ( $param_def->%* ) {
-        $self->_validate( $params, $param_def );
+
+    my ($validated_params, @errors) = VM::HetznerCloud::Schema->validate(
+        $opts->{oid},
+        $params,
+    );
+
+    $validated_params ||= {};
+
+    if ( @errors ) {
+        croak 'Invalid parameters';
     }
 
     my $method = delete $opts->{type} // 'get';
@@ -42,30 +50,28 @@ sub _request ( $self, $partial_uri, $params = {}, $param_def = {}, $opts = {} )
         croak sprintf 'Invalid request method %s', $method;
     }
 
-    $partial_uri =~ s{:(?<mandatory>\w+)\b}{ delete $params->{$+{mandatory}} }xmsge;
+    $partial_uri ||= '';
+    $partial_uri =~ s{:(?<mandatory>\w+)\b}{ ( delete $validated_params->{path}->{$+{mandatory}} ) // '' }xmsge;
+    $partial_uri =~ s{\A/}{};
+
+    my $query = join '&', map{
+        $_ . '=' . url_escape(  delete $validated_params->{query}->{$_} )
+    } grep {
+        $param_def->{$_}->{in} eq 'query' && $validated_params->{query}->{$_}
+    }sort keys $param_def->%*;
 
     my %request_opts;
     if ( $params->%* ) {
-        %request_opts = ( json => $params );
+        %request_opts = ( json => $validated_params->{body} );
     }
 
-    $opts->{query} //= {};
-    my $query = '';
-    if ( $opts->{query}->%* ) {
-        my $query_params = delete $opts->{query};
-
-        $query = join '&', map{ 
-            $_ . '=' . uri_escape($query_params->{$_}) 
-        }sort keys $query_params->%*;
-    }
-
-    $partial_uri .= '?' . $query if $query;
-
     my $uri = join '/', 
         $self->host, 
         $self->base_uri,
         $self->endpoint,
-        $partial_uri;
+        ( $partial_uri ? $partial_uri : () );
+
+    $uri .= '?' . $query if $query;
 
     my $tx = $self->client->$method(
         $uri,
@@ -84,26 +90,6 @@ sub _request ( $self, $partial_uri, $params = {}, $param_def = {}, $opts = {} )
     return $response->json;
 }
 
-sub _validate ( $self, $params, $def ) {
-    for my $param ( sort keys $def->%* ) {
-        if ( $def->{$param}->{required} && !length $params->{$param} ) {
-            die "Mandatory parameter $param not set!";
-        }
-
-        if ( $def->{$param}->{validation} ) {
-            my $error = $self->_validate_param( $params->{$param}, $def->{$param}->{validation} );
-        }
-    }
-
-    return 1;
-}
-
-sub _validate_param ( $self, $value, $format ) {
-    return if !$value;
-
-    return;
-}
-
 1;
 
 __END__
@@ -118,7 +104,7 @@ VM::HetznerCloud::APIBase - Base class for all entity classes
 
 =head1 VERSION
 
-version 0.0.2
+version 0.0.3
 
 =head1 AUTHOR
 
